/***********************************************************
*  Aquarius II -- McDuffins Underwater Missions            *
*----------------------------------------------------------*
*  1995 Artsoft Development                               *
*        Holger Schemel                                    *
*        33659 Bielefeld-Senne                             *
*        Telefon: (0521) 493245                            *
*        eMail: aeglos@valinor.owl.de                      *
*               aeglos@uni-paderborn.de                    *
*               q99492@pbhrzx.uni-paderborn.de             *
*----------------------------------------------------------*
*  events.c                                                *
*                                                          *
*  Letzte Aenderung: 15.06.1995                            *
***********************************************************/

#include "events.h"
#include "screens.h"
#include "tools.h"
#include "game.h"
#include "editor.h"

void EventLoop(void)
{
  while(1)
  {
    if (XPending(display))	/* got an event */
    {
      XEvent event;

      XNextEvent(display, &event);

      switch(event.type)
      {
	case Expose:
	  HandleExposeEvent((XExposeEvent *) &event);
	  break;
	case UnmapNotify:
	  SleepWhileUnmapped();
	  break;
	case ButtonPress:
	  button_status = ((XButtonEvent *) &event)->button;
	  HandleButtonEvent((XButtonEvent *) &event);
	  break;
	case ButtonRelease:
	  button_status = MB_RELEASED;
	  HandleButtonEvent((XButtonEvent *) &event);
	  break;
	case MotionNotify:
	  HandleMotionEvent((XMotionEvent *) &event);
	  break;
	case KeyPress:
	  key_status = KEY_PRESSED;
	  HandleKeyEvent((XKeyEvent *) &event);
	  break;
	case KeyRelease:
	  key_status = KEY_RELEASED;
	  HandleKeyEvent((XKeyEvent *) &event);
	  break;
	case FocusIn:
	  HandleFocusEvent(FOCUS_IN);
	  break;
	case FocusOut:
	  HandleFocusEvent(FOCUS_OUT);
	  break;
	default:
	  break;
      }
    }
    else			/* got no event, but don't be lazy... */
    {
      HandleNoXEvent();

      if (game_status!=PLAYING)
	Delay(10000);		/* don't use all CPU time when idle */
    }

    if (game_status==EXITGAME)
      return;
  }
}

void ClearEventQueue()
{
  while(XPending(display))
  {
    XEvent event;

    XNextEvent(display, &event);

    switch(event.type)
    {
      case Expose:
        HandleExposeEvent((XExposeEvent *) &event);
	break;
      case UnmapNotify:
	SleepWhileUnmapped();
	break;
      case ButtonRelease:
	button_status = MB_RELEASED;
	break;
      case KeyRelease:
	key_status = KEY_RELEASED;
	break;
      case FocusIn:
	HandleFocusEvent(FOCUS_IN);
	break;
      case FocusOut:
	HandleFocusEvent(FOCUS_OUT);
	break;
      default:
	break;
    }
  }
}

void SleepWhileUnmapped()
{
  BOOL window_unmapped = TRUE;

  XAutoRepeatOn(display);

  while(window_unmapped)
  {
    XEvent event;

    XNextEvent(display, &event);

    switch(event.type)
    {
      case Expose:
        HandleExposeEvent((XExposeEvent *) &event);
	break;
      case ButtonRelease:
	button_status = MB_RELEASED;
	break;
      case KeyRelease:
	key_status = KEY_RELEASED;
	break;
      case MapNotify:
	window_unmapped = FALSE;
	break;
      default:
	break;
    }
  }

  if (game_status==PLAYING)
    XAutoRepeatOff(display);
}

void HandleExposeEvent(XExposeEvent *event)
{
  int x=event->x, y=event->y;
  int w=event->width, h=event->height;

  XCopyArea(display,pix[PIX_DB_BACK],window,gc,x,y,w,h,x,y);
}

void HandleButtonEvent(XButtonEvent *event)
{
  HandleButton(event->x, event->y, button_status);
}

void HandleMotionEvent(XMotionEvent *event)
{
  if (game_status!=PLAYING)
    HandleButton(event->x, event->y, button_status);
}

void HandleKeyEvent(XKeyEvent *event)
{
  if (key_status==KEY_PRESSED)
    HandleKey(XLookupKeysym(event,event->state));
}

void HandleFocusEvent(int focus_status)
{
  if (focus_status==FOCUS_OUT)
    XAutoRepeatOn(display);
  else if (game_status==PLAYING)
    XAutoRepeatOff(display);
}

void HandleButton(int mx, int my, int button)
{
  static int old_mx=0, old_my=0;
  int delay;

  if (mx<0 || my<0)
  {
    mx=old_mx;
    my=old_my;
  }
  else
  {
    old_mx=mx;
    old_my=my;
  }

  switch(game_status)
  {
    case MAINMENU:
      HandleMainMenu(mx,my,0,0,button);
      break;
    case CHOOSENAME:
      HandleChooseName(mx,my,0,0,button);
      break;
    case HALLOFFAME:
      HandleHallOfFame(button);
      break;
    case LEVELED:
      LevelEd(mx,my,button);
      break;
    case HELPSCREEN:
      HandleHelpScreen(button);
      break;
    case SETUP:
      HandleSetupScreen(mx,my,0,0,button);
      break;
    case PLAYING:
      delay=Counter();
      WasserSteigt();

      if (!LevelSolved)
      {
	switch(GameActions(mx,my,button))
	{
	  case ACT_GAME_OVER:
	    game_status=MAINMENU;
	    DrawMainMenu();
	    BackToFront();
	    break;
	  case ACT_NEW_GAME:
	    game_status=PLAYING;
	    InitGame();
	    break;
	  case ACT_GO_ON:
	    break;
	  default:
	    break;
	}
      }
      if (Counter() <= delay+4)
	WaitCounter(delay+4);
      break;
    default:
      break;
  }
}

void HandleKey(KeySym key)
{
  static KeySym old_key = 0;

  if (!key)
    key = old_key;
  else
    old_key = key;

  if (key==XK_Escape && game_status!=MAINMENU)	/* quick quit to MAINMENU */
  {
    game_status=MAINMENU;
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      DX,DY, 100,280, DX,DY);
    redraw_mask|=REDRAW_DOOR;
    DR=0;
    DrawMainMenu();
    BackToFront();
    return;
  }

  switch(game_status)
  {
    case TYPENAME:
      HandleTypeName(0,key);
      break;
    case MAINMENU:
    {
      int dx=0, dy=0;

      switch(key)
      {
	case XK_Return:
	  HandleMainMenu(0,0,0,0,MB_MENU_CHOICE);
	  break;
	case XK_Left:
	case XK_KP_Left:
	case XK_KP_4:
	case XK_J:
	case XK_j:
	  dx=-1;
	  break;
	case XK_Right:
	case XK_KP_Right:
	case XK_KP_6:
	case XK_K:
	case XK_k:
	  dx=1;
	  break;
	case XK_Up:
	case XK_KP_Up:
	case XK_KP_8:
	case XK_I:
	case XK_i:
	  dy=-1;
	  break;
	case XK_Down:
	case XK_KP_Down:
	case XK_KP_2:
	case XK_M:
	case XK_m:
	  dy=1;
	  break;
	default:
	  break;
      }

      if (dx || dy)
	HandleMainMenu(0,0,dx,dy,MB_MENU_MARK);
      break;
    }
    case CHOOSENAME:
    {
      int dx=0, dy=0;

      switch(key)
      {
	case XK_Return:
	  HandleChooseName(0,0,0,0,MB_MENU_CHOICE);
	  break;
	case XK_Left:
	case XK_KP_Left:
	case XK_KP_4:
	case XK_J:
	case XK_j:
	  dx=-1;
	  break;
	case XK_Right:
	case XK_KP_Right:
	case XK_KP_6:
	case XK_K:
	case XK_k:
	  dx=1;
	  break;
	case XK_Up:
	case XK_KP_Up:
	case XK_KP_8:
	case XK_I:
	case XK_i:
	  dy=-1;
	  break;
	case XK_Down:
	case XK_KP_Down:
	case XK_KP_2:
	case XK_M:
	case XK_m:
	  dy=1;
	  break;
	default:
	  break;
      }

      if (dx || dy)
	HandleChooseName(0,0,dx,dy,MB_MENU_MARK);
      break;
    }
    case SETUP:
    {
      int dx=0, dy=0;

      switch(key)
      {
	case XK_Return:
	  HandleSetupScreen(0,0,0,0,MB_MENU_CHOICE);
	  break;
	case XK_Left:
	case XK_KP_Left:
	case XK_KP_4:
	case XK_J:
	case XK_j:
	  dx=-1;
	  break;
	case XK_Right:
	case XK_KP_Right:
	case XK_KP_6:
	case XK_K:
	case XK_k:
	  dx=1;
	  break;
	case XK_Up:
	case XK_KP_Up:
	case XK_KP_8:
	case XK_I:
	case XK_i:
	  dy=-1;
	  break;
	case XK_Down:
	case XK_KP_Down:
	case XK_KP_2:
	case XK_M:
	case XK_m:
	  dy=1;
	  break;
	default:
	  break;
      }

      if (dx || dy)
	HandleSetupScreen(0,0,dx,dy,MB_MENU_MARK);
      break;
    }
    case HALLOFFAME:
    case HELPSCREEN:
      switch(key)
      {
	case XK_Return:
	  game_status=MAINMENU;
	  DrawMainMenu();
	  BackToFront();
	  break;
	default:
	  break;
      }
      break;
    case LEVELED:
      break;
    case PLAYING:
    {
      int mvx=0, mvy=0;
      int sbx=0, sby=0;

      switch(key)
      {
	case XK_Left:		/* normale Richtungen */
	case XK_KP_Left:
	case XK_KP_4:
	case XK_J:
	case XK_j:
	  mvx=-1;
	  break;
	case XK_Right:
	case XK_KP_Right:
	case XK_KP_6:
	case XK_K:
	case XK_k:
	  mvx=1;
	  break;
	case XK_Up:
	case XK_KP_Up:
	case XK_KP_8:
	case XK_I:
	case XK_i:
	  mvy=-1;
	  break;
	case XK_Down:
	case XK_KP_Down:
	case XK_KP_2:
	case XK_M:
	case XK_m:
	  mvy=1;
	  break;
	case XK_KP_Home:	/* Diagonalrichtungen */
	case XK_KP_7:
	  mvx=-1;
	  mvy=-1;
	  break;
	case XK_KP_Page_Up:
	case XK_KP_9:
	  mvx=1;
	  mvy=-1;
	  break;
	case XK_KP_End:
	case XK_KP_1:
	  mvx=-1;
	  mvy=1;
	  break;
	case XK_KP_Page_Down:
	case XK_KP_3:
	  mvx=1;
	  mvy=1;
	  break;
	case XK_S:		/* Feld entfernen */
	case XK_s:
	  sbx=-1;
	  break;
	case XK_D:
	case XK_d:
	  sbx=1;
	  break;
	case XK_E:
	case XK_e:
	  sby=-1;
	  break;
	case XK_X:
	case XK_x:
	  sby=1;
	  break;
	default:
	  break;
      }

      SetBlock(JX+sbx,JY+sby);
      MoveFigure(mvx,mvy);

      break;
    }
    default:
      break;
  }
}

void HandleNoXEvent()
{
  int delay;

  if (button_status)
  {
    HandleButton(-1,-1,button_status);
    return;
  }

  switch(game_status)
  {
    case MAINMENU:
    case CHOOSENAME:
    case HALLOFFAME:
    case HELPSCREEN:
    case SETUP:
      HandleJoystick();
      break;
    case PLAYING:
      delay=Counter();
      WasserSteigt();
      HandleJoystick();
      if (key_status)
	HandleKey(0);
      if (game_status!=PLAYING)
	break;

      if (!LevelSolved)
      {
	switch(GameActions(0,0,MB_NOT_PRESSED))
	{
	  case ACT_GAME_OVER:
	    game_status=MAINMENU;
	    DrawMainMenu();
	    BackToFront();
	    break;
	  case ACT_NEW_GAME:
	    game_status=PLAYING;
	    InitGame();
	    break;
	  case ACT_GO_ON:
	    break;
	  default:
	    break;
	}
      }
      if (Counter() <= delay+4)
	WaitCounter(delay+4);
      break;
    default:
      break;
  }
}

void HandleJoystick()
{
  int joy	= Joystick();
  int left	= joy & JOY_LEFT;
  int right	= joy & JOY_RIGHT;
  int up	= joy & JOY_UP;
  int down	= joy & JOY_DOWN;
  int button	= joy & JOY_BUTTON;
  int newbutton	= (JoystickButton()==JOY_BUTTON_NEW_PRESSED);

  if (button_status || key_status)
    return;

  switch(game_status)
  {
    case MAINMENU:
    case CHOOSENAME:
    case SETUP:
    {
      int dx=0, dy=0;
      static long joystickmove_delay=0;

      if (DelayReached(&joystickmove_delay,15) || button)
      {
	if (left)
	  dx=-1;
	else if (right)
	  dx=1;
	if (up)
	  dy=-1;
	else if (down)
	  dy=1;
      }

      if (game_status==MAINMENU)
	HandleMainMenu(0,0,dx,dy,newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
      else if (game_status==CHOOSENAME)
	HandleChooseName(0,0,dx,dy,newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
      else if (game_status==SETUP)
	HandleSetupScreen(0,0,dx,dy,newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
      break;
    }
    case HALLOFFAME:
      HandleHallOfFame(!newbutton);
      break;
    case HELPSCREEN:
      HandleHelpScreen(!newbutton);
      break;
    case PLAYING:
    {
      static int block_was_just_set=FALSE;
      int mvx=0, mvy=0;

      if (left)
	mvx=-1;
      else if (right)
	mvx=1;
      if (up)
	mvy=-1;
      else if (down)
	mvy=1;

      if (button)
      {
        if (!block_was_just_set)
          block_was_just_set = SetBlock(JX+mvx,JY+mvy);
      }
      else
      {
        block_was_just_set = FALSE;
        MoveFigure(mvx,mvy);
      }

      break;
    }
    default:
      break;
  }
}
