/***********************************************************
*  Aquarius II -- McDuffins Underwater Missions            *
*----------------------------------------------------------*
*  1995 Artsoft Development                               *
*        Holger Schemel                                    *
*        33659 Bielefeld-Senne                             *
*        Telefon: (0521) 493245                            *
*        eMail: aeglos@valinor.owl.de                      *
*               aeglos@uni-paderborn.de                    *
*               q99492@pbhrzx.uni-paderborn.de             *
*----------------------------------------------------------*
*  game.c                                                  *
*                                                          *
*  Letzte Aenderung: 15.06.1995                            *
***********************************************************/

#include "game.h"
#include "misc.h"
#include "tools.h"
#include "screens.h"
#include "sound.h"
#include "events.h"

BOOL CreateNewLevelFile()
{
  int i,j;
  FILE *file;

  if (!(file=fopen(LEVEL_FILE,"w")))
    return(FALSE);

  for(i=0;i<100;i++)
  {
    fputc(3,file);	/* Figur */
    for(j=1;j<FIELDX*FIELDY-1;j++) 
      fputc(0,file);
    fputc(11,file);	/* Ausgang */
    fputc(100,file);	/* Zeit */
    for(j=1;j<16;j++) 	/* Rest */
      fputc(0,file);
  }
  fclose(file);
  return(TRUE);
}

BOOL CreateNewScoreFile()
{
  int i,j;
  FILE *file;

  if (!(file=fopen(SCORE_FILE,"w")))
    return(FALSE);

  for(i=0;i<100;i++)
  {
    for(j=0;j<MAX_PLAYERS;j++)
    {
      fprintf(file,"%10s ",EMPTY_ALIAS);
      fprintf(file,"%04d ",0);
    }
  }
  fclose(file);
  return(TRUE);
}

BOOL CreateNewNamesFile()
{
  int i;
  FILE *file;

  if (!(file=fopen(NAMES_FILE,"w")))
    return(FALSE);

  fprintf(file,"%s\n",NAMES_COOKIE);	/* Formatkennung */
  for(i=0;i<MAX_PLAYERS;i++)
  {
    fprintf(file,"%s \t",EMPTY_LOGIN);	/* Login-Name */
    fprintf(file,"%s \t",EMPTY_ALIAS);	/* Alias-Name */
    fprintf(file,"%d \t",0);		/* Active-Flag */
    fprintf(file,"%d \t",0);		/* Handicap */
    fprintf(file,"%x \n",DEFAULT_SETUP);/* Setup-Bits */
  }
  fclose(file);
  return(TRUE);
}

void LoadLevel(int level_nr)
{
  int x,y;
  FILE *file;

  if (!(file=fopen(LEVEL_FILE,"r")))
  {
    if (!CreateNewLevelFile())
    {
      fprintf(stderr,"%s: cannot create level file '%s'!\n",
	      progname,LEVEL_FILE);
    }
    else if (!(file=fopen(LEVEL_FILE,"r"))) 
    {
      fprintf(stderr,"%s: cannot load level %d!\n",
	      progname,level_nr);
    }
  }

  if (file)
  {
    fseek(file,level_nr*1040,SEEK_SET);
    for(y=0;y<FIELDY;y++) 
      for(x=0;x<FIELDX;x++) 
	Ur[x][y] = fgetc(file);
    leveltime = fgetc(file);
    fclose(file);
  }
  else
  {
    for(y=0;y<FIELDY;y++) 
      for(x=0;x<FIELDX;x++) 
	Ur[x][y] = 0;
    Ur[0][0] = 3;
    Ur[FIELDX-1][FIELDY-1] = 11;
    leveltime = 100;
  }

  actual_level = level_nr;
}

void LoadScore(int level_nr)
{
  int i;
  FILE *file;

  if (!(file=fopen(SCORE_FILE,"r")))
  {
    if (!CreateNewScoreFile())
    {
      fprintf(stderr,"%s: cannot create score file '%s'!\n",
	      progname,SCORE_FILE);
    }
    else if (!(file=fopen(SCORE_FILE,"r"))) 
    {
      fprintf(stderr,"%s: cannot load score for level %d!\n",
	      progname,level_nr);
    }
  }

  if (file)
  {
    fseek(file,level_nr*(MAX_PLAYERS*(MAX_NAMELEN+5)),SEEK_SET);
    for(i=0;i<MAX_PLAYERS;i++)
    {
      fscanf(file,"%s",highscore[i].Name);
      fscanf(file,"%d",&highscore[i].Score);
    }
    fclose(file);
  }
  else
  {
    for(i=0;i<MAX_PLAYERS;i++)
    {
      strcpy(highscore[i].Name,EMPTY_ALIAS);
      highscore[i].Score = 0;
    }
  }
}

void LoadNames()
{
  int i;
  char cookie[256];
  FILE *file;

  if (!(file=fopen(NAMES_FILE,"r")))
  {
    if (!CreateNewNamesFile())
    {
      fprintf(stderr,"%s: cannot create names file '%s'!\n",
	      progname,NAMES_FILE);
    }
    else if (!(file=fopen(NAMES_FILE,"r"))) 
    {
      fprintf(stderr,"%s: cannot load names!\n",
	      progname);
    }
  }

  if (file)
  {
    fscanf(file,"%s",cookie);
    if (strcmp(cookie,NAMES_COOKIE))	/* ungltiges Format? */
    {
      fprintf(stderr,"%s: wrong format of names file!\n",progname);
      fclose(file);
      file = NULL;
    }
  }

  if (file)
  {
    for(i=0;i<MAX_PLAYERS;i++)
    {
      fscanf(file,"%s",playerlist[i].login_name);
      fscanf(file,"%s",playerlist[i].alias_name);
      fscanf(file,"%d",&playerlist[i].last_used);
      fscanf(file,"%d",&playerlist[i].handicap);
      fscanf(file,"%x",&playerlist[i].setup);
    }
    fclose(file);
  }
  else
  {
    for(i=0;i<MAX_PLAYERS;i++)
    {
      strcpy(playerlist[i].login_name,EMPTY_LOGIN);
      strcpy(playerlist[i].alias_name,EMPTY_ALIAS);
      playerlist[i].last_used = FALSE;
      playerlist[i].handicap = 0;
      playerlist[i].setup = DEFAULT_SETUP;
    }
  }
}

void SaveLevel(int level_nr)
{
  int x,y;
  FILE *file;

  if (!(file=fopen(LEVEL_FILE,"r+")))
  {
    fprintf(stderr,"%s: cannot save level %d!\n",
	    progname,level_nr);
    return;
  }

  fseek(file,level_nr*1040,SEEK_SET);
  for(y=0;y<FIELDY;y++) 
    for(x=0;x<FIELDX;x++) 
      fputc(Ur[x][y],file);
  fputc(leveltime,file);
  fclose(file);
}

void SaveScore(int level_nr)
{
  int i;
  FILE *file;

  if (!(file=fopen(SCORE_FILE,"r+")))
  {
    fprintf(stderr,"%s: cannot save score for level %d!\n",
	    progname,level_nr);
    return;
  }

  fseek(file,level_nr*(MAX_PLAYERS*(MAX_NAMELEN+5)),SEEK_SET);
  for(i=0;i<MAX_PLAYERS;i++)
  {
    fprintf(file,"%10s ",highscore[i].Name);
    fprintf(file,"%04d ",highscore[i].Score);
  }
  fclose(file);
}

void SaveNames()
{
  int i;
  FILE *file;

  if (!(file=fopen(NAMES_FILE,"w")))
  {
    fprintf(stderr,"%s: cannot save names!\n",
	    progname);
    return;
  }

  fprintf(file,"%s\n",NAMES_COOKIE);	/* Formatkennung */
  for(i=0;i<MAX_PLAYERS;i++)
  {
    fprintf(file,"%s \t",playerlist[i].login_name);
    fprintf(file,"%s \t",playerlist[i].alias_name);
    fprintf(file,"%d \t",playerlist[i].last_used);
    fprintf(file,"%d \t",playerlist[i].handicap);
    fprintf(file,"%x \n",playerlist[i].setup);
  }
  fclose(file);
}

void GetPlayerName()
{
  int i;
  int player_pos, empty_pos, last_used;
  char *login_name;

  LoadNames();
  login_name = GetLoginName();
  player_pos = empty_pos = last_used = -1;

  for(i=0;i<MAX_PLAYERS;i++)
  {
    if (!strcmp(playerlist[i].login_name,login_name) && player_pos<0)
      player_pos = i;
    else if (!strcmp(playerlist[i].login_name,EMPTY_LOGIN) && empty_pos<0)
      empty_pos = i;
    if (playerlist[i].last_used)
      last_used = i;
  }

  if (player_pos<0 && empty_pos<0)	/* Liste voll, Spieler nicht drin */
    player = &playerlist[0];
  else if (player_pos>=0)		/* Spieler bereits in der Liste */
    player = &playerlist[player_pos];
  else					/* Spieler noch nicht in der Liste */
  {
    strncpy(playerlist[empty_pos].login_name,login_name,MAX_NAMELEN);
    strncpy(playerlist[empty_pos].alias_name,login_name,MAX_NAMELEN);
    playerlist[empty_pos].handicap = 0;
    SaveNames();
    player = &playerlist[empty_pos];
  }

  if (last_used>=0 && !strcmp(playerlist[last_used].login_name,login_name))
    player = &playerlist[last_used];

  level = player->handicap;
}

void GetPlayerConfig()
{
  if (sound_status==SOUND_OFF)
    player->setup &= ~SETUP_SOUND;
  if (!sound_loops_allowed)
  {
    player->setup &= ~SETUP_SOUND_LOOPS;
    player->setup &= ~SETUP_SOUND_MUSIC;
  }

  sound_on = SETUP_SOUND_ON(player->setup);
  sound_loops_on = SETUP_SOUND_LOOPS_ON(player->setup);
  sound_music_on = SETUP_SOUND_MUSIC_ON(player->setup);
  toons_on = SETUP_TOONS_ON(player->setup);
}

void DrawLevel()
{
  int x,y;

  if (level!=actual_level) 
    LoadLevel(level);

  ClearWindow();

  for(x=0;x<FIELDX;x++)
    for(y=0;y<FIELDY;y++)
      DrawElement(x,y,Ray[x][y]=Ur[x][y]);

  BackToFront();
}

void InitGame()
{
  int i,x,y;

  DrawLevel();

  NewLevel=Menu=Playing=Gems=Keys=Score=AYBits=PipeCnt=AqNew=cx=cy=0;
  NoStone=NoSource=-(LOK=Air=1);
  AirVol=100;


  EC=Ec=OC=Oc=PC=Pc=CT=Ct=0; DO=1;


  LevelSolved=0;
  Playing=1;

  for(x=0;x<32;x++)
  {
    AXBits[x]=ANBits[x]=0;
    for(y=0;y<32;y++)
      Aqua[x][y]=Stop[x][y]=Pipe[x][y]=Drop[x][y]=Blob[x][y]=0;
  }

  for(x=0;x<FIELDX;x++) for(y=0;y<FIELDY;y++)
  {
    switch(Ray[x][y])
    {
      case 3:		/* Figur */
	Ray[x][y]=0;
	JX=x;
	JY=y;
	break;
      case 4:		/* Edelstein */
	Gems++;
	break;
      case 5:		/* Schlssel */
	Keys++;
	break;
      default:
	break;
    }
  }

  XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_DOOR],gc, 300,0, 100,280, 300,0);
  DrawTextExt(pix[PIX_DB_DOOR],gc,300+glx,gly,int2str(level,2),
	      FS_SMALL,FC_YELLOW);
  DrawTextExt(pix[PIX_DB_DOOR],gc,300+gbx,gby,int2str(Gems,3),
	      FS_SMALL,FC_YELLOW);
  DrawTextExt(pix[PIX_DB_DOOR],gc,300+gsx,gsy,int2str(Score,4),
	      FS_SMALL,FC_YELLOW);
  OpenDoor(3);

  if (sound_loops_on)
    PlaySoundExt(SND_FUEL,PSND_MAX_VOLUME,PSND_MAX_RIGHT,PSND_LOOP);

  XCopyArea(display,pix[PIX_DOOR],drawto,gc, 
	    200+gex,geey-AirVol+2, 32,AirVol, GEX,GEEY-AirVol+2);

  for(i=2;i<=AirVol;i+=2)
  {
    if (!sound_loops_on)
      PlaySoundStereo(SND_FUEL,PSND_MAX_RIGHT);

    XCopyArea(display,pix[PIX_DOOR],window,gc, 
	      200+gex,geey-i, 32,i, GEX,GEEY-i);
    XFlush(display);
    ColorCycling();
    Delay(20000);
  }

  if (sound_loops_on)
    StopSound(SND_FUEL);

  if (sound_music_on)
    PlaySoundLoop(SND_TYGER);

  for(x=0;x<FIELDX;x++) for(y=0;y<FIELDY;y++)
  {
    if (Ray[x][y]==2)	/* offene Wasserquelle */
    {
      Ray[x][y]=0;
      WasserFliesst(x,y,4);
      WasserSteigt();
    }
    HandleJoystick();
  }

  XAutoRepeatOff(display);
}

void GameWon()
{
  int hi_pos;
  int bumplevel = FALSE;

  StopSound(1);

  XCopyArea(display,drawto,pix[PIX_DB_DOOR],gc, 
	    DX,DY, DXSIZE,DYSIZE, 300,0);
  CloseDoor(3);

  if (level==player->handicap && level<50) 
  { 
    player->handicap++; 
    bumplevel = TRUE;
    SaveNames(); 
  }

  if ((hi_pos=NewHiScore())>=0) 
  {
    game_status=HALLOFFAME;
    DrawHallOfFame(hi_pos);
    if (bumplevel)
      level++;
  }
  else
  {
    game_status=MAINMENU;
    if (bumplevel)
      level++;
    DrawMainMenu();
  }
  BackToFront();
}

BOOL NewHiScore()
{
  int position=-1;

  LoadScore(level);

  if (!strcmp(player->alias_name,EMPTY_ALIAS) || Score<highscore[9].Score) 
    return(-1);

  for(k=0;k<10;k++) 
  {
    if (Score>highscore[k].Score)
    {
      if (k<9) for(l=9;l>k;l--)
      {
	strcpy(highscore[l].Name,highscore[l-1].Name);
	highscore[l].Score=highscore[l-1].Score;
      }

      sprintf(highscore[k].Name,player->alias_name);

      highscore[k].Score=Score; 
      position=k;
      break;
    }
  }
  if (position>=0) 
    SaveScore(level);
  return(position);
}

void ColorCycling(void)
{
  static int CC,Cc=0;
  static int cycle=0;
  static unsigned short rgb[6][3] =
  {
    0x0000,0x6FFF,0xFFFF,
    0x0000,0x0000,0xFFFF,
    0x0000,0x0000,0xCFFF,
    0x0000,0x6FFF,0xFFFF,
    0x0000,0x0000,0xFFFF,
    0x0000,0x0000,0xCFFF
  };

  if (color_status==STATIC_COLORS)
    return;

  CC=Counter();

  if (CC<Cc || CC>Cc+5)
  {
    Cc=CC;
    cycle = (cycle + 1) % 3;
    SetRGB(pen_magicolor[0],rgb[cycle+0][0],rgb[cycle+0][1],rgb[cycle+0][2]);
    SetRGB(pen_magicolor[1],rgb[cycle+1][0],rgb[cycle+1][1],rgb[cycle+1][2]);
    SetRGB(pen_magicolor[2],rgb[cycle+2][0],rgb[cycle+2][1],rgb[cycle+2][2]);
  }
}

int GameActions(int mx, int my, int button)	/* was: WaitCounter() */
{
  int keine_luft;

  EC=OC=PC=CT=Counter();

  if (mx>DX+34 && mx<DX+64 && my>DY+122 && my<DY+152 && button)
  {
    if (!AreYouSure("Do you really want to quit the game ?",-3))
    { 
      OpenDoor(3); 
      return(0); 
    }
    else return(ACT_GAME_OVER);
  }

  if (mx>DX && my>DY+126 && my<DY+148 && button)
  {
    if (mx>DX+6 && mx<DX+28)
    { 
      sound_music_on = FALSE;
      FadeSound(SND_TYGER);
    }
    else if (mx>DX+70 && mx<DX+92)
    { 
      sound_music_on = TRUE;
      PlaySoundLoop(SND_TYGER);
    }  
  }

  keine_luft = (UNDERWATER(JX,JY) || Ray[JX][JY]==EL_SCHLEUSE);

  if (((keine_luft && AirVol>0) || (!keine_luft && AirVol<100)) 
      && (OC-Oc)>(3+keine_luft*5))
  {
    Oc=OC;
    if (keine_luft)	/* Luft sinkt, CO2 steigt */
    {
      int NoAirVol;

      AirVol--;
      NoAirVol=100-AirVol;
/*
      PlaySound(1,SND_WARNTON,0);
*/
      XCopyArea(display,pix[PIX_DOOR],drawto,gc, 
		300+gex,gey, 32,NoAirVol, GEX,GEY);
      XCopyArea(display,pix[PIX_DOOR],window,gc, 
		300+gex,gey, 32,NoAirVol, GEX,GEY);
      XCopyArea(display,pix[PIX_DOOR],drawto,gc, 
		200+gox,goey-NoAirVol, 32,NoAirVol, GOX,GOEY-NoAirVol);
      XCopyArea(display,pix[PIX_DOOR],window,gc, 
		200+gox,goey-NoAirVol, 32,NoAirVol, GOX,GOEY-NoAirVol);
      XFlush(display);
    }
    else		/* Luft steigt, CO2 sinkt */
    {
      AirVol++;
      StopSound(1);
      XCopyArea(display,pix[PIX_DOOR],drawto,gc, 
		200+gex,geey-AirVol, 32,AirVol, GEX,GEEY-AirVol);
      XCopyArea(display,pix[PIX_DOOR],window,gc, 
		200+gex,geey-AirVol, 32,AirVol, GEX,GEEY-AirVol);
      XCopyArea(display,pix[PIX_DOOR],drawto,gc, 
		300+gox,goy, 32,AirVol, GOX,GOY);
      XCopyArea(display,pix[PIX_DOOR],window,gc, 
		300+gox,goy, 32,AirVol, GOX,GOY);
      XFlush(display);
    }

    if (!AirVol)
    {
      StopSound(1); 
      return(AreYouSure("Mc Duffin out of oxygen ! Play it again ?",-3)+1);
    }
  }

  return(ACT_GO_ON);
}

void WasserFliesst(int qx, int qy, int type)
{
  int qx2[2], ChPps[16];
  int qt=2,aq,inaq=0,stop,fill=0,xx,yy,pp,PpCnt=0;

  if (!Playing || Ray[qx][qy]>6 || Aqua[qx][qy]<0)
    return;

  if (type)	/* Wasser fliet senkrecht hinunter */
  {
    while(qy<32 && Ray[qx][qy]<7 && Aqua[qx][qy]>=0)
    {
      DrawWater(8*qx,8*qy++,type);
      if (type==4)
	type=3;
    }
    qy--;
  }

  if (type && Aqua[qx][qy])
    return;

  NoStone=qy;
  qx2[0]=qx2[1]=xx=qx;
  if (!type || type==3)
    aq=RND(2);
  else
    aq=type-1;

  while(qt)	/* Wasser fliet links und rechts weg */
  {
    stop=0;
    if (qx>=0 && qx<32 && Ray[qx][qy]<7 && Aqua[qx][qy]>=0)
    {
      if (qy==31 || Ray[qx][qy+1]>6 || Aqua[qx][qy+1]<0)	/* Noch auf */
      {								/* Grund ? */
	if (!Aqua[qx][qy] && !(ANBits[qy]&LONGBITNR(qx)))
	{
	  if (qx<xx || (qx==xx && type==2))
	    pp=-3;
	  else if (qx>xx || (qx==xx && type==1))
	    pp=-1;
	  else
	    pp=-2;
	  DrawWater(8*qx,8*qy+7,pp);
	}
	if (qy<31 && Aqua[qx][qy+1]<0)			/* Auf Wasser */
	{
	  inaq=1;
	  if (!type)
	    Stop[qx][qy]=Pipe[qx][qy+1];
	  if (!PpCnt || Pipe[qx][qy+1]!=ChPps[PpCnt-1])
	    ChPps[PpCnt++]=Pipe[qx][qy+1];
	}
      }
      else
	WasserFliesst(qx,qy+(stop=1),2-aq);
    }
    else
      fill+=(stop=1);
    if (qt==2)
      aq=!aq;
    if (stop)
      qt--;
    if (qt)
      qx=qx2[aq]+=(aq*2-1);
  }

  if (type && inaq)
    return;

  if (fill<2 && PpCnt)
    for(pp=0;pp<PpCnt;pp++)
    {
      PipeState[ChPps[pp]]=1;
      for(xx=0;xx<FreeCnt;xx++)
	if (FreePipe[xx]==ChPps[pp])
	  FreePipe[xx]=0;
    }

  if (fill==2)		/* Wasser sammelt sich in einem Becken */
  {
    if (PpCnt>1)
    {
      for(yy=0;yy<32;yy++)
      {
	for(xx=0;xx<32;xx++) for(pp=0;pp<PpCnt;pp++)
	{
	  if (Pipe[xx][yy]==ChPps[pp])
	    Pipe[xx][yy]=ChPps[0];
	  if (Stop[xx][yy]==ChPps[pp])
	    Stop[xx][yy]=ChPps[0];
	}
      }
      pp=ChPps[0];
    }
    else if (PpCnt)
      pp=ChPps[0];
    else
      pp=++PipeCnt;

    PipeState[pp]=0;

    if (type)
      AYBits|=LONGBITNR(qy);
    else
      AqNew=1;
    for(xx=qx2[0]+1;xx<qx2[1];xx++)
    {
      DrawWater(8*xx,8*qy+7,0);
      if (type)
      {
	AXBits[qy]|=LONGBITNR(xx);
	Aqua[xx][qy]=1;
      }
      else
	ANBits[qy]|=LONGBITNR(xx);
      Pipe[xx][qy]=pp;
      Stop[xx][qy]=0;
    }
    for(xx=0;xx<32;xx++)
      if (Stop[xx][qy]==pp)
	PipeState[pp]=1;
    if (PipeState[pp]>0)
      for(i=0;i<FreeCnt;i++)
	if (FreePipe[i]==pp)
	  FreePipe[i]=0;
    if (!type && !PipeState[pp])
      PipeState[FreePipe[FreeCnt++]=pp]=-1;
  }
  NoStone=-1;

  BackToFront();
}

void WasserSteigt()
{
  int full=0;
  static long FC,Fc=0;

  FC=Counter();
  if ((FC<Fc+8 && FC>=Fc) || !Playing || !AYBits)
    return;
  Fc=FC;

  for(y=31;y>=0;y--)
  {
    NoSource=y;

    if (AYBits&LONGBITNR(y))
    {
      for(x=0;x<32;x++) if (AXBits[y]&LONGBITNR(x))
      {
	if (Aqua[x][y]<0)
	{
	  if (y && Ray[x][y-1]<7)
	  {
	    if (full<=0)
	    {
	      DrawWater(8*x,8*y-1,0);
	      ANBits[y-1]|=LONGBITNR(x);
	      Pipe[x][y-1]=Pipe[x][y];
	      full=-(AqNew=1);
	    }
	    else if (x==31 || !Aqua[x+1][y] || Ray[x+1][y-1]>6)
	      WasserFliesst(x,y-1,0);
	  }
	  if (full>=0 || !y)
	  {
	    AXBits[y]&=(LONGBITNR(x)^0xFFFFFFFF);
	    if (!AXBits[y])
	      AYBits&=(LONGBITNR(y)^0xFFFFFFFF);
	  }
	}
	else if ((!full && !PipeState[Pipe[x][y]]) || Aqua[x][y]>1)
	{
	  Aqua[x][y]++;
	  DrawWater(8*x,8*(y+1)-Aqua[x][y],0);
	  if (Aqua[x][y]==8)
	    Aqua[x][y]=-1;
	}
      }
/*
      if (full=full<0)
	y++;
*/

      full = (full<0);

      if (full)
	y++;

    }
  }
  if (FreeCnt)
    for(i=0;i<FreeCnt;i++)
      PipeState[FreePipe[i]]=0;
  if (AqNew) for(y=0;y<32;y++)
  {
    if (ANBits[y]) for(x=0;x<32;x++) if (ANBits[y]&LONGBITNR(x))
    {
      Aqua[x][y]=1;
      AXBits[y]|=LONGBITNR(x);
      AYBits|=LONGBITNR(y);
    }
    ANBits[y]=0;
  }
  FreeCnt=AqNew=0;
  NoSource=-1;

  BackToFront();
}

void RestoreWaterFlood(int x, int y)
{
  if (Aqua[x][y]>=0 && Blob[x][y])
  {
    static int restore[7] = { 4,3,1,2,-2,-3,-1 };
    int i;

    for(i=0;i<7;i++)
    {
      if (Blob[x][y] & (1<<i))
	DrawWater(8*x,8*y+7*(i>3),restore[i]);
    }
  }
}

void RestoreWaterPool(int x, int y)
{
  if (ABS(Aqua[x][y])>0)
  {
    int i,a=Aqua[x][y];

    if (a<0)
      a=8;

    for(i=0;i<a;i++)
      DrawWater(8*x,8*y+7-i,0);

    if (Aqua[x][y]<0)
      Blob[x][y]=0;
  }
}

void RestoreWater(int x, int y)
{
  RestoreWaterFlood(x,y);
  RestoreWaterPool(x,y);
}

int MoveFigureOneStep(int dx, int dy)
{
  int oldJX,oldJY, newJX=JX+dx,newJY=JY+dy;
  int element;

  if (!dx && !dy)
    return(FALSE);

  if (newJX<0 || newJX>=FIELDX || newJY<0 || newJY>=FIELDY)
    return(FALSE);

  element = Ray[newJX][newJY];

  if (element && element!=EL_EDELSTEIN && element!=EL_SCHLUESSEL
      && element!=EL_SCHLEUSE && (element!=EL_AUSGANG || Gems || Keys))
    return(FALSE);

  oldJX = JX;
  oldJY = JY;
  JX = newJX;
  JY = newJY;

  if (Ray[oldJX][oldJY]==EL_SCHLEUSE)
    DrawElement(oldJX,oldJY,EL_SCHLEUSE);
  else
    DrawGraphic(oldJX,oldJY,-1);

  RestoreWater(oldJX,oldJY);

  if (element==EL_EDELSTEIN || element==EL_SCHLUESSEL)
  {
    Ray[JX][JY]=0;
    DrawGraphic(JX,JY,-1);
    RestoreWater(JX,JY);

    if (element==EL_EDELSTEIN)
    {
      Gems--;
      Score+=10;
      DrawText(GBX,GBY,int2str(Gems,3),FS_SMALL,FC_YELLOW);
      DrawText(GSX,GSY,int2str(Score,4),FS_SMALL,FC_YELLOW);

      if (!WET(JX,JY) && !UNDERWATER(JX,JY))
      {
	DrawElement(JX,JY,EL_WASSER);
	WasserFliesst(JX,JY,4);
      }
    }
    else if (element==EL_SCHLUESSEL)
    {
      Keys--;
      Score+=10;
      DrawText(GSX,GSY,int2str(Score,4),FS_SMALL,FC_YELLOW);
    }
  }
  else if (element==EL_AUSGANG)
    LevelSolved=1;

  DrawElementThruMask(JX,JY,EL_FIGUR);

  return(TRUE);
}

void MoveFigure(int dx, int dy)
{
  static int count=0;
  int COUNT;

  if (!dx && !dy)
    return;

  COUNT=Counter();
  if (COUNT>count+6)
    count=COUNT;
  else
    return;

  if (MoveFigureOneStep(dx,0))
    MoveFigureOneStep(0,dy);
  else
  {
    MoveFigureOneStep(0,dy);
    MoveFigureOneStep(dx,0);
  }

  BackToFront();

  if (LevelSolved)
    GameWon();
}

int SetBlock(int x, int y)
{
  if (x<0 || x>=FIELDX || y<0 || y>=FIELDY)
    return(FALSE);

  if ((x==JX && y==JY) || Ray[x][y] || WET(x,y) || FLOOD(x,y))
    return(FALSE);

  Ray[x][y] = EL_HOLZWAND;
  DrawElement(x,y,EL_HOLZWAND);
  BackToFront();

  return(TRUE);
}
