/***********************************************************
*  Aquarius II -- McDuffins Underwater Missions            *
*----------------------------------------------------------*
*  1995 Artsoft Development                               *
*        Holger Schemel                                    *
*        33659 Bielefeld-Senne                             *
*        Telefon: (0521) 493245                            *
*        eMail: aeglos@valinor.owl.de                      *
*               aeglos@uni-paderborn.de                    *
*               q99492@pbhrzx.uni-paderborn.de             *
*----------------------------------------------------------*
*  events.c                                                *
*                                                          *
*  Letzte Aenderung: 15.06.1995                            *
***********************************************************/

#include "events.h"
#include "screens.h"
#include "tools.h"
#include "game.h"
#include "editor.h"

void EventLoop(void)
{
  while(1)
  {
    if (XPending(display))	/* got an event */
    {
      XEvent event;

      XNextEvent(display, &event);

      switch(event.type)
      {
	case Expose:
	  HandleExposeEvent((XExposeEvent *) &event);
	  break;
	case ButtonPress:
	  button_status = ((XButtonEvent *) &event)->button;
	  HandleButtonEvent((XButtonEvent *) &event);
	  break;
	case ButtonRelease:
	  button_status = MB_UNPRESSED;
	  HandleButtonEvent((XButtonEvent *) &event);
	  break;
	case MotionNotify:
	  HandleMotionEvent((XMotionEvent *) &event);
	  break;
	case KeyPress:
	case KeyRelease:
	  HandleKeyEvent((XKeyEvent *) &event);
	  break;
	default:
	  break;
      }
    }
    else			/* got no event, but don't be lazy... */
      HandleButton(-1,-1,button_status);

    if (game_status==EXITGAME)
      return;
  }
}

void HandleExposeEvent(XExposeEvent *event)
{
  int x=event->x, y=event->y;
  int w=event->width, h=event->height;

  XCopyArea(display,pix[PIX_DB_BACK],window,gc,x,y,w,h,x,y);
}

void HandleButtonEvent(XButtonEvent *event)
{
  HandleButton(event->x, event->y, button_status);
}

void HandleMotionEvent(XMotionEvent *event)
{
  if (game_status!=PLAYING)
    HandleButton(event->x, event->y, button_status);
}

void HandleButton(int mx, int my, int button)
{
  static int old_mx=0, old_my=0;
  int delay;

  if (mx<0 || my<0)
  {
    mx=old_mx;
    my=old_my;
  }
  else
  {
    old_mx=mx;
    old_my=my;
  }

  ColorCycling();

  switch(game_status)
  {
    case MAINMENU:
      HandleMainMenu(mx,my,button);
      if (!button)
	HandleJoystick();
      break;
    case CHOOSENAME:
      HandleNameInput(mx,my,button);
      if (!button)
	HandleJoystick();
      break;
    case HALLOFFAME:
      HandleHallOfFame(button);
      break;
    case LEVELED:
      LevelEd(mx,my,button);
      break;
    case HELPSCREEN:
      HandleHelpScreen(button);
      break;
    case PLAYING:
      delay=Counter();
      WasserSteigt();
      HandleJoystick();

      if (!Gotcha)
      {
	switch(GameActions(mx,my,button))
	{
	  case ACT_GAME_OVER:
	    game_status=MAINMENU;
	    DrawMainMenu();
	    BackToFront();
	    break;
	  case ACT_NEW_GAME:
	    game_status=PLAYING;
	    InitGame();
	    break;
	  case ACT_GO_ON:
	    break;
	  default:
	    break;
	}
      }

      if (!button && !OL && delay<=Counter())
	WaitCounter(delay+5);
      break;
    default:
      Delay(1000);
      break;
  }
}

void HandleKeyEvent(XKeyEvent *event)
{
  KeySym key = XLookupKeysym(event,event->state);

  switch(game_status)
  {
    case TYPENAME:
      HandleNameTyping(0,key);
      break;
    case MAINMENU:
      switch(key)
      {
	case XK_S:
	case XK_s:
	  sound_on=!sound_on;
	  break;
	case XK_T:
	case XK_t:
	  toons_status=!toons_status;
	  break;
	case XK_Return:
	  if (*AkName)
	  {
	    game_status=PLAYING;
	    InitGame();
	  }
	  break;
	default:
	  break;
      }
      break;
    case HALLOFFAME:
    case HELPSCREEN:
      switch(key)
      {
	case XK_Return:
	case XK_Escape:
	  game_status=MAINMENU;
	  DrawMainMenu();
	  BackToFront();
	  break;
	default:
	  break;
      }
      break;
    case PLAYING:
    {
      int mvx=0, mvy=0;
      int sbx=0, sby=0;

      switch(key)
      {
	case XK_Escape:
	  game_status=MAINMENU;
	  XCopyArea(display,pix[PIX_BACK],drawto,gc,
		    DX,DY, 100,280, DX,DY);
	  redraw_mask|=REDRAW_DOOR;
	  DR=0;
	  DrawMainMenu();
	  BackToFront();
	  break;
	case XK_Left:		/* normale Richtungen */
	case XK_KP_Left:
	case XK_KP_4:
	case XK_J:
	case XK_j:
	  mvx=-1;
	  break;
	case XK_Right:
	case XK_KP_Right:
	case XK_KP_6:
	case XK_K:
	case XK_k:
	  mvx=1;
	  break;
	case XK_Up:
	case XK_KP_Up:
	case XK_KP_8:
	case XK_I:
	case XK_i:
	  mvy=-1;
	  break;
	case XK_Down:
	case XK_KP_Down:
	case XK_KP_2:
	case XK_M:
	case XK_m:
	  mvy=1;
	  break;
	case XK_KP_Home:	/* Diagonalrichtungen */
	case XK_KP_7:
	  mvx=-1;
	  mvy=-1;
	  break;
	case XK_KP_Page_Up:
	case XK_KP_9:
	  mvx=1;
	  mvy=-1;
	  break;
	case XK_KP_End:
	case XK_KP_1:
	  mvx=-1;
	  mvy=1;
	  break;
	case XK_KP_Page_Down:
	case XK_KP_3:
	  mvx=1;
	  mvy=1;
	  break;
	case XK_S:		/* Block setzen */
	case XK_s:
	  sbx=-1;
	  break;
	case XK_D:
	case XK_d:
	  sbx=1;
	  break;
	case XK_E:
	case XK_e:
	  sby=-1;
	  break;
	case XK_X:
	case XK_x:
	  sby=1;
	  break;
	default:
	  break;
      }

      SetBlock(JX+sbx,JY+sby);
      MoveFigure(mvx,mvy);

      break;
    }
    default:
      break;
  }
}

void HandleJoystick()
{
  int newbutton=(JoystickButton()==JOY_BUTTON_NEW_PRESSED);
  int joy=Joystick();
  int left=joy & JOY_LEFT;
  int right=joy & JOY_RIGHT;
  int up=joy & JOY_UP;
  int down=joy & JOY_DOWN;
  int button=joy & JOY_BUTTON;

  if (!joy)
    return;

  switch(game_status)
  {
    case MAINMENU:
    {
      static int xpos=1,ypos=3;
      static int count=0;
      int COUNT=Counter();

      if (COUNT>=count && COUNT<=count+10)
	return;

      count=COUNT;

      if (left)
      {
	if (ypos==4)
	  xpos=11;
      }
      else if (right)
      {
	if (ypos==4)
	  xpos=14;
      }

      if (up && ypos>3)
      {
	xpos=1;
	ypos--;
      }
      else if (down && ypos<10)
      {
	xpos=1;
	ypos++;
      }

      if (newbutton)
      {
	HandleMainMenu(SX+32*(xpos-1),SY+32*(ypos-1),TRUE);
	HandleMainMenu(SX+32*(xpos-1),SY+32*(ypos-1),FALSE);
      }
      else
      {
	HandleMainMenu(SX+32*(xpos-1),SY+32*(ypos-1),TRUE);
	HandleMainMenu(0,0,FALSE);
      }

      break;
    }
    case CHOOSENAME:
    {
      static int xpos=1,ypos=1;
      static int count=0;
      int COUNT=Counter();

      if (COUNT>=count && COUNT<=count+10)
	return;

      count=COUNT;

      if (up && ypos>1)
      {
	ypos--;
	if (ypos==11)
	  ypos=10;
      }
      else if (down && ypos<12)
      {
	ypos++;
	if (ypos==11)
	  ypos=12;
      }

      if (newbutton)
      {
	HandleNameInput(SX+32*(xpos-1),SY+32*(ypos-1),TRUE);
	HandleNameInput(SX+32*(xpos-1),SY+32*(ypos-1),FALSE);
      }
      else
      {
	HandleNameInput(SX+32*(xpos-1),SY+32*(ypos-1),TRUE);
	HandleNameInput(0,0,FALSE);
      }

      break;
    }
    case PLAYING:
    {
      static int block_was_just_set=FALSE;
      int mvx=0, mvy=0;

      if (left)
	mvx=-1;
      else if (right)
	mvx=1;
      if (up)
	mvy=-1;
      else if (down)
	mvy=1;

      if (button)
      {
	if (!block_was_just_set)
	  block_was_just_set = SetBlock(JX+mvx,JY+mvy);
      }
      else
      {
	block_was_just_set = FALSE;
	MoveFigure(mvx,mvy);
      }

      break;
    }
    default:
      break;
  }
}
