/***********************************************************
*  Aquarius II -- McDuffins Underwater Missions            *
*----------------------------------------------------------*
*  1995 Artsoft Development                               *
*        Holger Schemel                                    *
*        33659 Bielefeld-Senne                             *
*        Telefon: (0521) 493245                            *
*        eMail: aeglos@valinor.owl.de                      *
*               aeglos@uni-paderborn.de                    *
*               q99492@pbhrzx.uni-paderborn.de             *
*----------------------------------------------------------*
*  tools.c                                                 *
*                                                          *
*  Letzte Aenderung: 15.06.1995                            *
***********************************************************/

#include "tools.h"
#include "game.h"
#include "events.h"
#include "sound.h"

void BackToFront()
{
  int x,y;

  if (!redraw_mask)
    return;
  else if (redraw_mask & REDRAW_ALL || 
	   (redraw_mask & REDRAW_FIELD && redraw_mask & REDRAW_DOOR))
    XCopyArea(display,pix[PIX_DB_BACK],window,gc,0,0,WIN_XSIZE,WIN_YSIZE,0,0);
  else if (redraw_mask & REDRAW_FIELD)
    XCopyArea(display,pix[PIX_DB_BACK],window,gc,6,6,SXSIZE+4,SYSIZE+4,6,6);
  else if (redraw_mask & REDRAW_DOOR)
    XCopyArea(display,pix[PIX_DB_BACK],window,gc,DX,DY,DXSIZE,DYSIZE,DX,DY);

  if (redraw_mask & REDRAW_TILES && 
      !(redraw_mask & REDRAW_FIELD) && !(redraw_mask & REDRAW_ALL))
  {
    if (redraw_tiles>REDRAWTILES_TH)
      XCopyArea(display,pix[PIX_DB_BACK],window,gc,SX,SY,SXSIZE,SYSIZE,SX,SY);
    else
      for(x=0;x<FIELDX;x++)
	for(y=0;y<FIELDY;y++)
	  if (redraw[x][y])
	    XCopyArea(display,pix[PIX_DB_BACK],window,gc,
		      SX+x*16,SY+y*16,16,16,SX+x*16,SY+y*16);
  }

  XFlush(display);
  for(x=0;x<FIELDX;x++)
    for(y=0;y<FIELDY;y++)
      redraw[x][y]=0;
  redraw_tiles=0;
  redraw_mask=0;
}

void ClearWindow()
{
  XFillRectangle(display,drawto,gc,SX-2,SY-2,SXSIZE+4,SYSIZE+4);
  redraw_mask|=REDRAW_FIELD;
}

void DrawText(int x, int y, char *text, int font, int col)
{
  DrawTextExt(drawto, gc, x, y, text, font, col);
  if (x<DX)
    redraw_mask|=REDRAW_FIELD;
  else
    redraw_mask|=REDRAW_DOOR;
}

void DrawTextExt(Drawable d, GC gc, int x, int y, char *text, int font,int col)
{
  if (font!=FS_SMALL && font!=FS_BIG)
    font=FS_SMALL;
  if (col<FC_RED || col>FC_YELLOW)
    col=FC_RED;
  for(;*text;text++,x+=(14+font*18))
  {
    int k=*text; 
    int l=-1;

    if (k>64 && k<91) l=k-65;
    if (k>96 && k<123) l=k-97;
    if (k>47 && k<58) l=k-18;
    if (k==63) l=26;
    if (k==33) l=27;
    if (k==58) l=28;
    if (k==46) l=29;

    if (l<0)
      XFillRectangle(display,d,gc,
		     x,y,14+(font==FS_BIG)*18,12+(font==FS_BIG)*20);
    else if (font==FS_BIG)
      XCopyArea(display,pix[PIX_BIGFONT],d,gc,
		(l%20)*32,(2*col+l/20)*32, 32,32, x,y);
    else
      XCopyArea(display,pix[PIX_SMALLFONT],d,gc,
		l*14,col*14, 14,12, x,y);
  }
}

void DrawGraphic(int x, int y, int g)
{
  DrawGraphicExt(drawto, gc, x, y, g);
  redraw_tiles++;
  redraw[x][y]=TRUE;
  redraw_mask|=REDRAW_TILES;
}

void DrawGraphicExt(Drawable d, GC gc, int x, int y, int g)
{
  if (g>=0)
    XCopyArea(display,pix[PIX_BACK],d,gc,
	      SX+(g%16)*32,SY+(g/16)*32, 32,32, SX+x*32,SY+y*32);
  else
    XFillRectangle(display,d,gc, SX+x*16,SY+y*16, 16,16);
}

void DrawGraphic32(int x, int y, int g)
{
  DrawGraphicExt32(drawto, gc, x, y, g);
  redraw_tiles+=4;
  redraw[2*x][2*y]=TRUE;
  redraw[2*x+1][2*y]=TRUE;
  redraw[2*x][2*y+1]=TRUE;
  redraw[2*x+1][2*y+1]=TRUE;
  redraw_mask|=REDRAW_TILES;
}

void DrawGraphicExt32(Drawable d, GC gc, int x, int y, int g)
{
  if (g>=0)
    XCopyArea(display,pix[PIX_BACK],d,gc,
	      SX+(g%16)*32,SY+(g/16)*32, 32,32, SX+x*32,SY+y*32);
  else
    XFillRectangle(display,d,gc, SX+x*32,SY+y*32, 32,32);
}

int el2gfx(int element)
{
  int graphic;

  if (!element)
    return(-1);
  else if (element==EL_WASSER)		graphic=GFX_WASSER;
  else if (element==EL_FIGUR)		graphic=GFX_FIGUR;
  else if (element==EL_EDELSTEIN)	graphic=GFX_EDELSTEIN;
  else if (element==EL_SCHLUESSEL)	graphic=GFX_SCHLUESSEL;
  else if (element==EL_GITTER)		graphic=GFX_GITTER;
  else if (element==EL_SCHLEUSE)	graphic=GFX_SCHLEUSE;
  else if (element==EL_STEINWAND)	graphic=GFX_STEINWAND;
  else if (element==EL_HOLZWAND)	graphic=GFX_HOLZWAND;
  else if (element==EL_AUSGANG)		graphic=GFX_AUSGANG;
  else
    return(-1);

  return(graphic);
}

void DrawElement(int xe, int ye, int element)
{
  int width=TILEX, height=TILEY;
  int graphic;

  if (!element)
    return;

  graphic = el2gfx(element);

  if (element==EL_WASSER)
    height=12;

  XCopyArea(display,pix[PIX_BACK],drawto,gc,
	    SX+graphic*16,SY+ELY, width,height, SX+xe*16,SY+ye*16);

  redraw_tiles++;
  redraw[xe][ye]=TRUE;
  redraw_mask|=REDRAW_TILES;
}

void DrawElementThruMask(int xe, int ye, int element)
{
  int src_x,src_y, sx,sy;
  int dest_x=SX+xe*16, dest_y=SY+ye*16;
  int width=TILEX, height=TILEY;
  int graphic, mask;
  int gdiff=10;

  if (!element)
    return;

  graphic = mask = el2gfx(element);

  if (element==EL_FIGUR || element==EL_EDELSTEIN || element==EL_SCHLUESSEL)
  {
    if (WET(xe,ye))
      mask+=gdiff;
    if (UNDERWATER(xe,ye))
      graphic+=gdiff;
  }

  src_x  = SX+graphic*16;
  src_y  = SY+ELY;
  sx     = mask*16;
  sy     = 0;

  XSetClipOrigin(display,clip_gc[PIX_OBJECTS],dest_x-sx,dest_y-sy);

  if ((element==EL_FIGUR || element==EL_EDELSTEIN || element==EL_SCHLUESSEL)
      && WET(xe,ye) && !UNDERWATER(xe,ye))
  {
    int water = 2*Aqua[xe][ye];
    int nowater = height-water;

    XCopyArea(display,pix[PIX_BACK],drawto,clip_gc[PIX_OBJECTS], src_x,src_y,
	      width,nowater, dest_x,dest_y);
    XCopyArea(display,pix[PIX_BACK],drawto,clip_gc[PIX_OBJECTS],
	      src_x+gdiff*16,src_y+nowater,
	      width,water, dest_x,dest_y+nowater);
  }
  else
    XCopyArea(display,pix[PIX_BACK],drawto,clip_gc[PIX_OBJECTS], src_x,src_y,
	      width,height, dest_x,dest_y);

  redraw_tiles++;
  redraw[xe][ye]=TRUE;
  redraw_mask|=REDRAW_TILES;
}

void DrawMicroElement(int xe, int ye, int e)
{
  int g;

  if (!e) return;
  g=el2gfx(e);

  XCopyArea(display,pix[PIX_BACK],drawto,gc,
            SX+g*4,SY+480, 4,4, SX+12*32+xe*4,SY+6*32+ye*4);
}

void DrawMicroLevel(int xpos, int ypos)
{
  int x,y;

  LoadLevel(level);
  XFillRectangle(display,drawto,gc, xpos,ypos, 4*32,4*32);

  for(x=0;x<FIELDX;x++) for(y=0;y<FIELDY;y++)
    DrawMicroElement(x,y,Ur[x][y]);

  XCopyArea(display,pix[PIX_DB_BACK],window,gc,
	    xpos,ypos, 4*32,4*32, xpos,ypos);
}

void DrawWater(int dwx, int dwy, int type)
{
  int wx=dwx/8,wy=dwy/8;

  if (wx<32)
    Drop[wx][wy]|=(1+(type>0));

  if (type==4)		/* am Anfang */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WASSER*16,SY+ELY, 16,16, SX+2*dwx,SY+2*dwy);
    Blob[wx][wy]|=1;
  }
  else if (type==3)	/* in der Mitte */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WFLOOD1*16+6,SY+ELY, 4,16, SX+2*dwx+6,SY+2*dwy);
    Blob[wx][wy]|=2;
  }
  else if (type==1)	/* links */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WFLOOD1*16,SY+ELY, 2,16, SX+2*dwx,SY+2*dwy);
    Blob[wx][wy]|=4;
  }
  else if (type==2)    	/* rechts */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WFLOOD1*16+14,SY+ELY, 2,16, SX+2*dwx+14,SY+2*dwy);
    Blob[wx][wy]|=8;
  }
  else if (type==-2)   	/* in der Mitte auftreffend */
  {
    if (Blob[wx][wy] & 2)
      XCopyArea(display,pix[PIX_BACK],drawto,gc,
		SX+GFX_WFLOOD2*16+2,SY+ELY+8, 12,6, SX+2*dwx+2,SY+2*dwy-6);
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WFLOOD2*16,SY+ELY+14, 16,2, SX+2*dwx,SY+2*dwy);
    Blob[wx][wy]|=16;
  }
  else if (type==-3)   	/* nach links flieend */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WLEFT*16,SY+ELY+14, 16,2, SX+2*dwx,SY+2*dwy);
    Blob[wx][wy]|=32;
  }
  else if (type==-1)   	/* nach rechts flieend */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WRIGHT*16,SY+ELY+14, 16,2, SX+2*dwx,SY+2*dwy);
    Blob[wx][wy]|=64;
  }
  else if (type==0)	/* steigendes Wasser */
  {
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WUP*16+RND(17),SY+ELY+14, 16,1, SX+2*dwx,SY+2*dwy);
    XCopyArea(display,pix[PIX_BACK],drawto,gc,
	      SX+GFX_WUP*16+RND(17),SY+ELY+15, 16,1, SX+2*dwx,SY+2*dwy+1);
  }
  else
    printf("HEH! Was ist denn 'type==%d'? (%d/%d)\n", type, wx,wy);

  if (type==1 || type==2 || type==3)
    RestoreWaterPool(wx,wy);

  if (Ray[wx][wy])
    DrawElementThruMask(wx,wy,Ray[wx][wy]);
  else if (wx==JX && wy==JY)
    DrawElementThruMask(wx,wy,EL_FIGUR);
  else
  {
    redraw_tiles++;
    redraw[wx][wy]=TRUE;
    redraw_mask|=REDRAW_TILES;
  }
}

int AYS_in_range(int x, int y)
{
  if (y>DY+249 && y<DY+278)
  {
    if (x>DX+1 && x<DX+48)
      return(1);
    else if (x>DX+51 && x<DX+98) 
      return(2);
  }
  return(0);
}

BOOL AreYouSure(char *text, int behind)
{
  int mx,my,tx,ty,tl,tc,rt=-1;
  int dr=DR;
  static char txt[100];
  BOOL choosen=-1;

  if (ABS(behind)==3)
    XCopyArea(display,pix[PIX_DB_BACK],pix[PIX_DB_DOOR],gc,
	      DX,DY, 100,280, 300,0);

  XFillRectangle(display,pix[PIX_DB_DOOR],gc,100,0,100,280);

  for(ty=0;ty<13;ty++)
  {
    if (!(*text))
      break;
    for(tl=0,tx=0;tx<7;tl++,tx++)
    {
      tc=*(text+tx);
      if (!tc || tc==32)
	break;
    }
    if (!tl)
    { 
      text++; 
      ty--; 
      continue; 
    }
    sprintf(txt,text); 
    txt[tl]=0;
    DrawTextExt(pix[PIX_DB_DOOR],gc,
		152-(tl*14)/2,SY+ty*16,txt,FS_SMALL,FC_YELLOW);
    text+=(tl+(tc==32));
  }
  if (!behind)
    return(FALSE);

  XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_DOOR],gc, 202,2, 96,28, 102,250);

  if (dr)
    CloseDoor(ABS(behind));
  OpenDoor(1);

  if (game_status!=MAINMENU)
    InitAnimation();

  while(rt<0)
  {
    ColorCycling();
    DoAnimation();
    Delay(10000);

    if (XPending(display))
    {
      XEvent event;

      XNextEvent(display, &event);
      switch(event.type)
      {
	case Expose:
	  HandleExposeEvent((XExposeEvent *) &event);
	  break;
	case ButtonPress:
	  mx = ((XButtonEvent *) &event)->x;
	  my = ((XButtonEvent *) &event)->y;
	  if (!(choosen=AYS_in_range(mx,my)))
	    choosen=-1;
	  break;
	case ButtonRelease:
	  button_status = MB_NOT_PRESSED;
	  mx = ((XButtonEvent *) &event)->x;
	  my = ((XButtonEvent *) &event)->y;
	  if (choosen==AYS_in_range(mx,my))
	    rt=(choosen==1);
	  break;
	case KeyPress:
	case KeyRelease:
	  switch(XLookupKeysym((XKeyEvent *)&event,
			       ((XKeyEvent *)&event)->state))
	  {
	    case XK_Return:
	      rt=1;
	      break;
	    case XK_Escape:
	      rt=0;
	      break;
	  }
	  break;
	default:
	  break;
      }
    }
    else if (JoystickButton()==JOY_BUTTON_NEW_PRESSED)
    {
      int joy=Joystick();

      if (joy & JOY_BUTTON_1)
	rt=1;
      else if (joy & JOY_BUTTON_2)
	rt=0;
    }
  }

  if (game_status!=MAINMENU)
    StopAnimation();

  XCopyArea(display,pix[PIX_DB_BACK],pix[PIX_DB_DOOR],gc,
	    DX,DY, 100,280, 100,0);
  CloseDoor(1);
  if (dr && behind>0)
    OpenDoor(behind);

  return(rt);
}

void MoveDoor(int nr, int dr)   /* dr==0: Tr zu, dr==1: Tr auf. */
{
  int i;

  if (dr)
  {
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DOOR],gc, 104,136, 8,8, 146,136);
    XCopyArea(display,pix[PIX_DOOR],window,gc, 104,136, 8,8, DX+46,DY+136);
    XFlush(display);
    for(i=0;i<30;i++)
    {
      ColorCycling();
      if (game_status==MAINMENU)
	DoAnimation();
      Delay(10000);
    }
  }
  else
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DOOR],gc, 88,136, 8,8, 146,136);

  PlaySoundStereo(SND_OEFFNEN,PSND_MAX_RIGHT);

  for(i=100*dr;i>=0 && i<=100;i=i+4-dr*8)
  {
    XCopyArea(display,pix[PIX_DB_DOOR],pix[PIX_DB_BACK],gc,
	      nr*100,i/2, 100,280-i/2, DX,DY);

    XFillRectangle(display,pix[PIX_DB_BACK],gc,DX,DY+280-i/2,100,i/2);

    XSetClipOrigin(display,clip_gc[PIX_DOOR],DX-100+i,DY);
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
	      100-i,0, i,30, DX,DY);
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
	      100-i,250, i,30, DX,DY+250);
    XSetClipOrigin(display,clip_gc[PIX_DOOR],DX-i,DY);
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
	      100,0, i,30, DX+100-i,DY);
    XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
	      100,250, i,30, DX+100-i,DY+250);
    if (i>14)
    {
      XSetClipOrigin(display,clip_gc[PIX_DOOR],DX-i,DY);
      XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
		114,30,i-14,220,DX+114-i,DY+30);
      XSetClipOrigin(display,clip_gc[PIX_DOOR],DX-100+i,DY);
      XCopyArea(display,pix[PIX_DOOR],pix[PIX_DB_BACK],clip_gc[PIX_DOOR],
		100-i,30,i-14,220,DX,DY+30);
    }

    redraw_mask|=REDRAW_DOOR;
    BackToFront();
    Delay(10000);

    ColorCycling();
    if (game_status==MAINMENU)
      DoAnimation();
  }

  DR=dr;
}

long mainCounter(int mode)
{
  static struct timeval base_time = { 0, 0 };
  struct timeval current_time;
  long counter_ms;

  gettimeofday(&current_time,NULL);
  if (mode==0 || current_time.tv_sec<base_time.tv_sec)
    base_time = current_time;

  counter_ms = (current_time.tv_sec - base_time.tv_sec)*1000
             + (current_time.tv_usec - base_time.tv_usec)/1000;

  if (mode==1)
    return(counter_ms/10);	/* return 1/100 secs since last init */
  else
    return(counter_ms);		/* return 1/1000 secs since last init */
}

void InitCounter() /* set counter back to zero */
{
  mainCounter(0);
}

long Counter()	/* returns 1/100 secs since last call of InitCounter() */
{
  return(mainCounter(1));
}

long Counter2()	/* returns 1/1000 secs since last call of InitCounter() */
{
  return(mainCounter(2));
}

void WaitCounter(long value) 	/* wait for counter to reach value */
{
  long wait;

  while((wait=value-Counter())>0)
    microsleep(wait*10000);
}

void WaitCounter2(long value) 	/* wait for counter to reach value */
{
  long wait;

  while((wait=value-Counter2())>0)
    microsleep(wait*1000);
}

void Delay(long value)
{
  microsleep(value);
}

BOOL DelayReached(long *counter_var, int delay)
{
  long actual_counter = Counter();

  if (actual_counter>*counter_var+delay || actual_counter<*counter_var)
  {
    *counter_var = actual_counter;
    return(TRUE);
  }
  else
    return(FALSE);
}

int ReadPixel(Drawable d, int x, int y)
{
  static XImage *pixelimage;

  pixelimage = XGetImage(display, d, x,y, 1,1, AllPlanes, ZPixmap);
  return(XGetPixel(pixelimage,0,0));
}

void SetRGB(unsigned long pixel, 
	    unsigned short red, unsigned short green, unsigned short blue)
{
  XColor color;

  if (color_status==STATIC_COLORS)
    return;

  color.pixel = pixel;
  color.red = red;
  color.green = green;
  color.blue = blue;
  color.flags = DoRed | DoGreen | DoBlue;
  XStoreColor(display, cmap, &color);
  XFlush(display);
}

static int joystick_xleft   = JOYSTICK_XLEFT;
static int joystick_xright  = JOYSTICK_XRIGHT;
static int joystick_xmiddle = JOYSTICK_XMIDDLE;
static int joystick_yupper  = JOYSTICK_YUPPER;
static int joystick_ylower  = JOYSTICK_YLOWER;
static int joystick_ymiddle = JOYSTICK_YMIDDLE;

void LoadJoystickData()
{
  FILE *file;

  if (joystick_status==JOYSTICK_OFF)
    return;

  if (!(file=fopen(JOYDAT_FILE,"r")))
    return;
  fscanf(file, "%d %d %d \n",
	  &joystick_xleft, &joystick_xmiddle, &joystick_xright);
  fscanf(file, "%d %d %d \n",
	  &joystick_yupper, &joystick_ymiddle, &joystick_ylower);
  fclose(file);

  CheckJoystickData();
}

void SaveJoystickData()
{
  FILE *file;

  if (joystick_status==JOYSTICK_OFF)
    return;

  CheckJoystickData();

  if (!(file=fopen(JOYDAT_FILE,"w")))
  {
    fprintf(stderr,"%s: cannot save joystick calibration data!\n",progname);
    return;
  }
  fprintf(file, "%d %d %d \n",
	  joystick_xleft, joystick_xmiddle, joystick_xright);
  fprintf(file, "%d %d %d \n",
	  joystick_yupper, joystick_ymiddle, joystick_ylower);
  fclose(file);
}

void CheckJoystickData()
{
  int distance = 100;

  if (joystick_xmiddle <= distance)
    joystick_xmiddle = distance;
  if (joystick_ymiddle <= distance)
    joystick_ymiddle = distance;

  if (joystick_xleft >= joystick_xmiddle)
    joystick_xleft = joystick_xmiddle-distance;
  if (joystick_xright <= joystick_xmiddle)
    joystick_xright = joystick_xmiddle+distance;

  if (joystick_yupper >= joystick_ymiddle)
    joystick_yupper = joystick_ymiddle-distance;
  if (joystick_ylower <= joystick_ymiddle)
    joystick_ylower = joystick_ymiddle+distance;
}

int JoystickPosition(int middle, int margin, int actual)
{
  long range, pos;
  int percentage;

  if (margin<middle && actual>middle)
    return(0);
  if (margin>middle && actual<middle)
    return(0);

  range=ABS(margin-middle);
  pos=ABS(actual-middle);
  percentage=(int)(pos*100/range);
  if (percentage>100)
    percentage=100;

  return(percentage);
}

int Joystick()
{
  struct joystick_control
  {
    int buttons;
    int x;
    int y;
  } joy_ctrl;

  int js_x,js_y, js_b1,js_b2;
  int left, right, up, down;
  int result=0;

  if (joystick_status==JOYSTICK_OFF)
    return(0);

  if (read(joystick_device, &joy_ctrl, sizeof(joy_ctrl)) != sizeof(joy_ctrl))
  {
    fprintf(stderr,"%s: cannot read joystick settings - no joystick support\n",progname);
    joystick_status=JOYSTICK_OFF;
    return(0);
  }

  js_x  = joy_ctrl.x;
  js_y  = joy_ctrl.y;
  js_b1 = joy_ctrl.buttons & 1;
  js_b2 = joy_ctrl.buttons & 2;

  left =  JoystickPosition(joystick_xmiddle, joystick_xleft,  js_x);
  right = JoystickPosition(joystick_xmiddle, joystick_xright, js_x);
  up =    JoystickPosition(joystick_ymiddle, joystick_yupper, js_y);
  down =  JoystickPosition(joystick_ymiddle, joystick_ylower, js_y);

  if (left>JOYSTICK_PERCENT)
    result |= JOY_LEFT;
  else if (right>JOYSTICK_PERCENT)
    result |= JOY_RIGHT;
  if (up>JOYSTICK_PERCENT)
    result |= JOY_UP;
  else if (down>JOYSTICK_PERCENT)
    result |= JOY_DOWN;
  if (js_b1)
    result |= JOY_BUTTON_1;
  if (js_b2)
    result |= JOY_BUTTON_2;

  return(result);
}

int JoystickButton()
{
  static int last_joy_button=0;
  int joy_button=(Joystick() & JOY_BUTTON);
  int result;

  if (joy_button)
  {
    if (last_joy_button)
      result=JOY_BUTTON_PRESSED;
    else
      result=JOY_BUTTON_NEW_PRESSED;
  }
  else
  {
    if (last_joy_button)
      result=JOY_BUTTON_NEW_RELEASED;
    else
      result=JOY_BUTTON_NOT_PRESSED;
  }

  last_joy_button = joy_button;
  return(result);
}

void CalibrateJoystick()
{
  struct joystick_control
  {
    int buttons;
    int x;
    int y;
  } joy_ctrl;

  int new_joystick_xleft, new_joystick_xright, new_joystick_xmiddle;
  int new_joystick_yupper, new_joystick_ylower, new_joystick_ymiddle;

  if (joystick_status==JOYSTICK_OFF)
    goto error_out;

  ClearWindow();
  DrawText(SX, SY+16+0*32, "MOVE JOYSTICK TO",FS_BIG,FC_YELLOW);
  DrawText(SX, SY+16+1*32, " THE UPPER LEFT ",FS_BIG,FC_YELLOW);
  DrawText(SX, SY+16+2*32, "AND PRESS BUTTON",FS_BIG,FC_YELLOW);
  BackToFront();

  joy_ctrl.buttons = 0;
  while(Joystick() & JOY_BUTTON);
  while(!joy_ctrl.buttons)
  {
    if (read(joystick_device, &joy_ctrl, sizeof(joy_ctrl)) != sizeof(joy_ctrl))
    {
      joystick_status=JOYSTICK_OFF;
      goto error_out;
    }
    Delay(10000);
  }

  new_joystick_xleft = joy_ctrl.x;
  new_joystick_yupper = joy_ctrl.y;

  ClearWindow();
  DrawText(SX,    SY+16+0*32, "MOVE JOYSTICK TO",FS_BIG,FC_YELLOW);
  DrawText(SX+16, SY+16+1*32, "THE LOWER RIGHT",FS_BIG,FC_YELLOW);
  DrawText(SX,    SY+16+2*32, "AND PRESS BUTTON",FS_BIG,FC_YELLOW);
  BackToFront();

  joy_ctrl.buttons = 0;
  while(Joystick() & JOY_BUTTON);
  while(!joy_ctrl.buttons)
  {
    if (read(joystick_device, &joy_ctrl, sizeof(joy_ctrl)) != sizeof(joy_ctrl))
    {
      joystick_status=JOYSTICK_OFF;
      goto error_out;
    }
    Delay(10000);
  }

  new_joystick_xright = joy_ctrl.x;
  new_joystick_ylower = joy_ctrl.y;

  ClearWindow();
  DrawText(SX+16, SY+16+0*32, "CENTER JOYSTICK",FS_BIG,FC_YELLOW);
  DrawText(SX,    SY+16+2*32, "AND PRESS BUTTON",FS_BIG,FC_YELLOW);
  BackToFront();

  joy_ctrl.buttons = 0;
  while(Joystick() & JOY_BUTTON);
  while(!joy_ctrl.buttons)
  {
    if (read(joystick_device, &joy_ctrl, sizeof(joy_ctrl)) != sizeof(joy_ctrl))
    {
      joystick_status=JOYSTICK_OFF;
      goto error_out;
    }
    Delay(10000);
  }

  new_joystick_xmiddle = joy_ctrl.x;
  new_joystick_ymiddle = joy_ctrl.y;

  joystick_xleft = new_joystick_xleft;
  joystick_yupper = new_joystick_yupper;
  joystick_xright = new_joystick_xright;
  joystick_ylower = new_joystick_ylower;
  joystick_xmiddle = new_joystick_xmiddle;
  joystick_ymiddle = new_joystick_ymiddle;

  CheckJoystickData();

  DrawSetupScreen();
  while(Joystick() & JOY_BUTTON);
  return;

  error_out:

  ClearWindow();
  DrawText(SX+16, SY+16, "NO JOYSTICK",FS_BIG,FC_YELLOW);
  DrawText(SX+16, SY+48, " AVAILABLE ",FS_BIG,FC_YELLOW);
  Delay(3000000);
  DrawSetupScreen();
}
